import greenfoot.Actor;
import greenfoot.GreenfootImage;

import java.awt.Color;
import java.awt.Font;
import java.util.Calendar;

/**
 * Representation of a clock.
 * 
 * @author Dietrich Boles (University of Oldenburg, Germany)
 * @version 01.12.2009
 */
public class Clock extends Actor {
	final static int SIZE = 94;
	private GreenfootImage image;
	private GreenfootImage clockImage;
	private int maxTime;
	private int imageTime;
	private boolean suspended;
	private boolean stopped;
	private long startTime;
	private long pauseStartTime;
	private int pauseDuration;
	private int penaltySeconds;
	
	public Clock() {
	    this(60);
	}

	public Clock(int maxTime) {
		this.suspended = false;
		this.stopped = false;
		this.maxTime = maxTime;
		this.startTime = 0;
		this.imageTime = maxTime;
		this.pauseStartTime = 0;
		this.pauseDuration = 0;
		this.penaltySeconds = 0;
		this.image = new GreenfootImage(Clock.SIZE, Clock.SIZE);
		this.image.setFont(new Font("Monospaced", Font.BOLD, 24));
		this.clockImage = new GreenfootImage("sun.gif");
		this.updateImage();
		this.setImage(this.image);
	}

	/**
	 * reset the clock
	 * 
	 * @param sec
	 *            start time
	 */
	public void reset() {
		this.suspended = false;
		this.stopped = false;
		this.pauseStartTime = 0;
		this.pauseDuration = 0;
		this.penaltySeconds = 0;
		this.updateImage();
	}

	/**
	 * gets the remaining time
	 * 
	 * @return the remaining time
	 */
	public int getRestTime() {
		return this.imageTime;
	}

	/**
	 * reduce the time
	 * 
	 * @param seconds
	 *            number of seconds to reduce
	 */
	public void reduce(int seconds) {
		this.penaltySeconds += seconds;
	}

	/**
	 * starts the clock
	 * 
	 * @param sec
	 *            start time
	 */
	public void start() {
		this.reset();
		this.startTime = Calendar.getInstance().getTimeInMillis();
		this.imageTime = this.maxTime;
		this.updateImage();
	}

	/**
	 * resume the clock
	 */
	public void resume() {
		this.suspended = false;
		long currentTime = Calendar.getInstance().getTimeInMillis();
		this.pauseDuration += (int) (currentTime - this.pauseStartTime);
	}

	/**
	 * suspend the clock
	 */
	public void suspend() {
		this.suspended = true;
		this.pauseStartTime = Calendar.getInstance().getTimeInMillis();
	}

	/**
	 * @return true, if the clock has been suspended
	 */
	public boolean isSuspended() {
		return this.suspended;
	}

	/**
	 * stop the clock
	 */
	public void stop() {
		this.stopped = true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see greenfoot.Actor#act()
	 */
	@Override
	public void act() {
		if (this.suspended || this.stopped) {
			return;
		}

		long currentTime = Calendar.getInstance().getTimeInMillis();
		this.imageTime = this.maxTime
				- ((int) (currentTime - this.startTime) - this.pauseDuration + this.penaltySeconds * 1000)
				/ 1000;
		if (this.imageTime < 0) {
			this.imageTime = 0;
		}
		this.updateImage();
		if (this.imageTime == 0) { // time has expired
			((WasteWorld) this.getWorld()).finish();
		}
	}

	/**
	 * draw the clock image
	 */
	private void updateImage() {
		this.image.drawImage(this.clockImage, 0, 0);
		String timeString = "" + this.imageTime;
		int slen = Metrics.stringWidth(timeString, this.image.getFont());
		this.image.setColor(new Color(50, 50, 50));
		this.image.drawString(timeString, Clock.SIZE / 2 - slen / 2, 53);
	}
}
