import greenfoot.Greenfoot;
import greenfoot.World;

import java.awt.Color;
import java.util.List;

/**
 * Representation of the "waste world"
 * 
 * @author Dietrich Boles (University of Oldenburg, Germany)
 * @version 01.12.2009
 */
public class WasteWorld extends World {
	final static int WIDTH = 656;
	final static int HEIGHT = 500;
	final static Color BG_COLOR = new Color(100, 100, 100);
	final static int PENALTY_SECONDS = 5; // making a wrong separation
	final static int SECONDS = 90;

	final static int NOOF_TYPES = 6;

	final static int REST = 0;
	final static int COMPOST = 1;
	final static int GLASS = 2;
	final static int METAL = 3;
	final static int PAPER = 4;
	final static int PLASTIC = 5;

	private Dustbin[] dustbin;
	private Dustbin openDustbin;
	private Waste[] wasteFactory;
	private Waste currentWaste;

	private Clock clock;
	private Counter counter;

	public WasteWorld() {
		super(WasteWorld.WIDTH, WasteWorld.HEIGHT, 1);
		this.setBackground("wasteworld.gif");
		this.dustbin = new Dustbin[WasteWorld.NOOF_TYPES];
		this.openDustbin = null;

		this.dustbin[WasteWorld.REST] = new DustbinRest();
		this.dustbin[WasteWorld.COMPOST] = new DustbinCompost();
		this.dustbin[WasteWorld.GLASS] = new DustbinGlass();
		this.dustbin[WasteWorld.METAL] = new DustbinMetal();
		this.dustbin[WasteWorld.PAPER] = new DustbinPaper();
		this.dustbin[WasteWorld.PLASTIC] = new DustbinPlastic();

		this.addObject(this.dustbin[WasteWorld.REST], 53,
				WasteWorld.HEIGHT - 135);
		this.addObject(this.dustbin[WasteWorld.COMPOST], 163,
				WasteWorld.HEIGHT - 135);
		this.addObject(this.dustbin[WasteWorld.GLASS], 273,
				WasteWorld.HEIGHT - 135);
		this.addObject(this.dustbin[WasteWorld.METAL], 383,
				WasteWorld.HEIGHT - 135);
		this.addObject(this.dustbin[WasteWorld.PAPER], 493,
				WasteWorld.HEIGHT - 135);
		this.addObject(this.dustbin[WasteWorld.PLASTIC], 603,
				WasteWorld.HEIGHT - 135);

		this.wasteFactory = new Waste[WasteWorld.NOOF_TYPES];
		this.wasteFactory[WasteWorld.REST] = new WasteRest();
		this.wasteFactory[WasteWorld.COMPOST] = new WasteCompost();
		this.wasteFactory[WasteWorld.GLASS] = new WasteGlass();
		this.wasteFactory[WasteWorld.METAL] = new WasteMetal();
		this.wasteFactory[WasteWorld.PAPER] = new WastePaper();
		this.wasteFactory[WasteWorld.PLASTIC] = new WastePlastic();

		this.clock = new Clock(WasteWorld.SECONDS);
		this.addObject(this.clock, WasteWorld.WIDTH - 60, 60);

		this.counter = new Counter();
		this.addObject(this.counter, 100, 72);

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see greenfoot.World#started()
	 */
	@Override
	public void started() {
		this.removeObjects(this.getObjects(Waste.class));
		this.removeObjects(this.getObjects(Points.class));
		this.removeObjects(this.getObjects(Correct.class));
		this.removeObjects(this.getObjects(Wrong.class));
		this.counter.reset();
		for (Dustbin d : this.dustbin) {
			d.setClosed();
		}
		this.openDustbin = null;

		Greenfoot.setSpeed(50);
		this.generateNewWaste();
		this.clock.start();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see greenfoot.World#stopped()
	 */
	@Override
	public void stopped() {
		this.clock.stop();
	}

	/**
	 * time limit exceeded
	 */
	public void finish() {
		Points points = new Points(this.counter.getValue());
		this.addObject(points, WasteWorld.WIDTH / 2, WasteWorld.HEIGHT / 3);
		Greenfoot.stop();
	}

	/**
	 * checks whether a piece of waste is touching a dustbin; if true the
	 * dustbin is opened
	 */
	public void checkWasteIntersection() {
		List iDustbins = this.currentWaste
				.getIntersectingObjects(Dustbin.class);
		if (iDustbins.size() == 0) {
			this.openDustbin = null;
		} else if (iDustbins.size() == 1) {
			this.openDustbin = (Dustbin) iDustbins.get(0);
		} else { // more than one (can only be two)
			if (this.openDustbin == null) {
				this.openDustbin = (Dustbin) iDustbins.get(0);
			}
		}

		for (int i = 0; i < WasteWorld.NOOF_TYPES; i++) {
			if (this.openDustbin == this.dustbin[i]) {
				this.dustbin[i].setOpen();
			} else {
				this.dustbin[i].setClosed();
			}
		}
	}

	/**
	 * checks whether a piece of waste has been thrown in the correct dustbin
	 */
	public void checkDustbinIntersection() {
		if (this.openDustbin != null) {
			if (this.openDustbin.getType() == this.currentWaste.getType()) {

				// correct dustbin
				this.addObject(new Correct(), this.openDustbin.getX(),
						WasteWorld.HEIGHT - 45);
				this.counter.add(1);
			} else {

				// wrong dustbin
				this.addObject(new Wrong(), this.openDustbin.getX(),
						WasteWorld.HEIGHT - 45);
				this.clock.reduce(WasteWorld.PENALTY_SECONDS);
			}
			this.removeObject(this.currentWaste);
			this.openDustbin.setClosed();
			this.openDustbin = null;
			this.generateNewWaste();
		}
	}

	/**
	 * checks whether the current piece of waste has reached the bottom
	 * 
	 * @return true, if the current piece of waste has reached the bottom
	 */
	public boolean checkWasteAtTheBottom() {
		if (this.currentWaste.getY() >= WasteWorld.HEIGHT - 1) {
			this.addObject(new Wrong(), this.dustbin[this.currentWaste
					.getType()].getX(), WasteWorld.HEIGHT - 45);
			this.clock.reduce(WasteWorld.PENALTY_SECONDS);
			if (this.openDustbin != null) {
				this.openDustbin.setClosed();
				this.openDustbin = null;
			}
			this.generateNewWaste();
			return true;
		} else {
			return false;
		}
	}

	/**
	 * set the current piece of waste to the bottom
	 */
	public void setToBottom() {
		this.currentWaste.setLocation(this.currentWaste.getX(),
				WasteWorld.HEIGHT - 1);
		if (this.openDustbin != null) {
			this.openDustbin.setClosed();
			this.openDustbin = null;
		}
		this.addObject(new Wrong(), this.dustbin[this.currentWaste.getType()]
				.getX(), WasteWorld.HEIGHT - 45);
		this.clock.reduce(WasteWorld.PENALTY_SECONDS);
		this.generateNewWaste();
	}

	/**
	 * generates a new piece of waste
	 */
	private void generateNewWaste() {
		int type = Greenfoot.getRandomNumber(WasteWorld.NOOF_TYPES);
		this.currentWaste = this.wasteFactory[type].createWaste();
		this.addObject(this.currentWaste, Greenfoot
				.getRandomNumber(WasteWorld.WIDTH - 100) + 50, 0);
	}

}
